#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */

#include <tarot.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>

#ifndef LOCALEDIR
#define LOCALEDIR "/usr/share/locale"
#endif /* NOT LOCALEDIR */

#ifndef DATADIR
#define DATADIR "/usr/share"
#endif /* NOT LOCALEDIR */

#define P1 0
#define P2 1
#define P3 2
#define P4 3
#define P5 4
#define DOG 5

static inline TarotCard
queen_of_diamonds (void)
{
  TarotCard c;
  if (tarot_of (TAROT_QUEEN, TAROT_DIAMONDS, &c) != 0)
    {
      abort ();
    }
  return c;
}

int
main ()
{
  TarotGame *game;
  TarotGameEvent *event;
  static const TarotPlayer owners[78] =
    {
     /* Hearts */
     /* A, 2,  3,   4,   5,   6,   7,   8,   9,   10,  J,   C,   Q,   K */
     P5,  P1,  P5,  P3,  DOG, P2,  P1,  P4,  P3,  P2,  P4,  P5,  P3,  P3,
     /* Clubs */
     DOG, P3,  P5,  P2,  P3,  P1,  P4,  P5,  P4,  P4,  P2,  P2,  DOG, P4,
     /* Diamonds */
     P2,  P1,  P3,  P1,  P4,  P4,  P5,  P3,  P3,  P2,  P4,  P2,  P4,  P3,
     /* Spades */
     P2,  P1,  P4,  P5,  P2,  P1,  P1,  P3,  P3,  P2,  P1,  P2,  P5,  P5,
     /* Trumps */
     P1,  P1,  P1,  P3,  P5,  P2,  P3,  P4,  P2,  P5,  P5,  P4,  P1,  P2,  P4,  P5,  P1,  P1,  P5,  P5,  P4,
     /* Excuse */
     P3
    };
  TarotGameError code;
  int i;

  tarot_set_datadir (DATADIR);  
  if (tarot_init (LOCALEDIR) != 0)
    {
      fprintf (stderr, "Error: could not initialize libtarot.\n");
      return EXIT_FAILURE;
    }
  game = tarot_game_alloc ();

  /* Set the duplicaté options early */
  code = tarot_game_duplicate_set_deal (game, 78, owners);
  assert (code == TAROT_GAME_OK);
  code = tarot_game_duplicate_set_taker (game, P2);
  assert (code == TAROT_GAME_OK);
  /* This is impossible, P2 does not have all kings...  It has none in
   * fact. */
  code = tarot_game_duplicate_set_call (game, queen_of_diamonds ());
  /* However, since game has not accepted the deal yet, it has no
   * reason to refuse. */
  assert (code == TAROT_GAME_OK); /* !!! */
  
  /* The game has 5 players, with call */
  event = tarot_game_event_alloc_setup (5, 1);
  if (tarot_game_add_event (game, event) != TAROT_GAME_OK)
    {
      abort ();
    }
  tarot_game_event_free (event);

  /* Now the game has jumped to the declaration phase */
  assert (tarot_game_step (game) == TAROT_DECLS);

  /* There are no problems for the first 4 declarations */
  for (i = 0; i < 4; i++)
    {
      event = tarot_game_event_alloc_decl (0);
      if (tarot_game_add_event (game, event) != TAROT_GAME_OK)
	{
	  abort ();
	}
      tarot_game_event_free (event);
    }

  /* Let's check the fifth. */
  event = tarot_game_event_alloc_decl (0);
  /* This event is valid. */
  if (!tarot_game_check_event (game, event))
    {
      abort ();
    }
  /* But we cannot add it! */
  if (tarot_game_add_event (game, event) != TAROT_GAME_INVEV)
    {
      abort ();
    }
  tarot_game_event_free (event);
  /* How is that?  The game has advanced to the call phase though! */
  assert (tarot_game_step (game) == TAROT_CALL);

  /* But the duplicaté call is refused. */
  tarot_game_free (game);
  tarot_quit ();
  return EXIT_SUCCESS;
}
