<?xml version="1.0" encoding="UTF-8"?>
<!-- tarot implements the rules of the tarot game
Copyright (C) 2018, 2019  Vivien Kraus

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
-->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
		xmlns:tc="http://planete-kraus.eu/tarot/cards"
		version="1.0">
  <xsl:output method="text" indent="no"/>
  <xsl:strip-space elements="*" />

  <xsl:template match="tc:cards">
    <xsl:text>/**&#xA; * file src/libtarot/cards/tarot/cards_defs.h&#xA; * This file has been generated from src/libtarot/cards/cards_defs.xsl.&#xA; */&#xA;#ifndef H_TAROT_CARDS_DEFS_INCLUDED&#xA;#define H_TAROT_CARDS_DEFS_INCLUDED&#xA;#include &lt;tarot/cards.h&gt;&#xA;</xsl:text>
    <xsl:apply-templates />
    <xsl:text>#endif /* not H_TAROT_CARDS_DEFS_INCLUDED */&#xA;</xsl:text>
  </xsl:template>

  <xsl:template match="tc:card">
    <xsl:text>#define TAROT_</xsl:text>
    <xsl:value-of select="@name" />
    <xsl:text> </xsl:text>
    <xsl:value-of select="@id" />
    <xsl:text>&#xA;</xsl:text>
  </xsl:template>

  <xsl:template mode="decompose" match="tc:card[boolean(@number) and boolean(@suit)]">
    <xsl:text>  if (c == </xsl:text>
    <xsl:value-of select="@id" />
    <xsl:text>)&#xA;    {&#xA;      *n = </xsl:text>
    <xsl:value-of select="@number" />
    <xsl:text>;&#xA;      *s = </xsl:text>
    <xsl:value-of select="@suit" />
    <xsl:text>;&#xA;      return 0;&#xA;    }&#xA;</xsl:text>
  </xsl:template>

  <xsl:template name="parse_code">
    <xsl:text>  if (len >= strlen (name_</xsl:text>
    <xsl:value-of select="@name" />
    <xsl:text>) &amp;&amp; strncmp (text, name_</xsl:text>
    <xsl:value-of select="@name" />
    <xsl:text>, strlen (name_</xsl:text>
    <xsl:value-of select="@name" />
    <xsl:text>)) == 0)&#xA;    {&#xA;      if (end != NULL)&#xA;        {&#xA;          *end = (char *) text + strlen (name_</xsl:text>
    <xsl:value-of select="@name" />
    <xsl:text>);&#xA;        }&#xA;      return </xsl:text>
    <xsl:value-of select="@id" />
    <xsl:text>;&#xA;    }&#xA;</xsl:text>
  </xsl:template>

  <xsl:template mode="card_parse" match="tc:card">
    <xsl:text>  char *name_</xsl:text>
    <xsl:value-of select="@name" />
    <xsl:text> = _ (&quot;</xsl:text>
    <xsl:value-of select="@name" />
    <xsl:text>&quot;);&#xA;</xsl:text>
    <xsl:call-template name="parse_code" />
  </xsl:template>

  <xsl:template mode="card_parse_c" match="tc:card">
    <xsl:text>  char *name_</xsl:text>
    <xsl:value-of select="@name" />
    <xsl:text> = &quot;</xsl:text>
    <xsl:value-of select="@name" />
    <xsl:text>&quot;;&#xA;</xsl:text>
    <xsl:call-template name="parse_code" />
  </xsl:template>

  <xsl:template name="to_string_code">
    <xsl:text>  if (c == </xsl:text>
    <xsl:value-of select="@id" />
    <xsl:text>)&#xA;    {&#xA;      size_t required = strlen (name_</xsl:text>
    <xsl:value-of select="@name" />
    <xsl:text>);&#xA;      if (max > required)&#xA;        {&#xA;          strncpy (dest, name_</xsl:text>
    <xsl:value-of select="@name" />
    <xsl:text>, required);&#xA;          dest[required] = '\0';&#xA;        }&#xA;      else if (max != 0)&#xA;        {&#xA;          strncpy (dest, name_</xsl:text>
    <xsl:value-of select="@name" />
    <xsl:text>, max - 1);&#xA;          dest[max - 1] = '\0';&#xA;        }&#xA;      return required;&#xA;    }&#xA;</xsl:text>
  </xsl:template>

  <xsl:template mode="card_to_string" match="tc:card">
    <xsl:text>  char *name_</xsl:text>
    <xsl:value-of select="@name" />
    <xsl:text> = _ (&quot;</xsl:text>
    <xsl:value-of select="@name" />
    <xsl:text>&quot;);&#xA;</xsl:text>
    <xsl:call-template name="to_string_code" />
  </xsl:template>

  <xsl:template mode="card_to_string_c" match="tc:card">
    <xsl:text>  char *name_</xsl:text>
    <xsl:value-of select="@name" />
    <xsl:text> = &quot;</xsl:text>
    <xsl:value-of select="@name" />
    <xsl:text>&quot;;&#xA;</xsl:text>
    <xsl:call-template name="to_string_code" />
  </xsl:template>
</xsl:stylesheet>
