/* tarot implements the rules of the tarot game
 * Copyright (C) 2019  Vivien Kraus
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>
#include <tarot/solo_private.h>
#include <string.h>
#include <assert.h>
#include <stdlib.h>
#include <stdalign.h>
#include "xalloc.h"

#ifdef __FRAMAC__
#undef alignof
#define alignof(x) (16)
#endif /* __FRAMAC__ */

TarotSolo *
tarot_solo_alloc (void)
{
  size_t alignment;
  size_t required = solo_construct (0, NULL, &alignment);
  size_t unused;
  void *mem = xmalloc (required);
  assert (alignment >= 1 && alignment <= alignof (max_align_t));
  unused = solo_construct (required, mem, &alignment);
  assert (unused == required);
  (void) unused;
  return mem;
}

TarotSolo *
tarot_solo_dup (const TarotSolo * source)
{
  TarotSolo *ret = tarot_solo_alloc ();
  tarot_solo_copy (ret, source);
  return ret;
}

void
tarot_solo_free (TarotSolo * solo)
{
  free (solo);
}

size_t
tarot_solo_construct (size_t max_mem, char *mem, size_t *alignment)
{
  return solo_construct (max_mem, mem, alignment);
}

void
tarot_solo_copy (TarotSolo * dest, const TarotSolo * source)
{
  solo_copy (dest, source);
}

void
tarot_solo_copy_as (TarotSolo * dest, const TarotSolo * source,
                    TarotPlayer who)
{
  solo_copy_as (dest, source, who);
}

void
tarot_solo_get (const TarotSolo * solo, TarotGame * game)
{
  solo_get (solo, game);
}

TarotGame *
tarot_solo_get_alloc (const TarotSolo * solo)
{
  TarotGame *ret = tarot_game_alloc ();
  solo_get (solo, ret);
  return ret;
}

TarotGameError
tarot_solo_setup (TarotSolo * solo, size_t n_players, int with_call,
                  TarotPlayer myself, size_t n_owners,
                  const TarotPlayer * owners, TarotPlayer taker,
                  TarotCard call)
{
  return solo_setup (solo, n_players, with_call, myself, n_owners, owners,
                     taker, call);
}

TarotGameError
tarot_solo_setup_random (TarotSolo * solo, size_t n_players, int with_call,
                         size_t seed_size, const void *seed)
{
  return solo_setup_random (solo, n_players, with_call, seed_size, seed);
}

TarotGameError
tarot_solo_add (TarotSolo * solo, const TarotGameEvent * event)
{
  return solo_add (solo, event);
}

#include <tarot/solo_private_impl.h>
